#!/usr/bin/env python3
# This script is part of Acronis Backup Advanced for SAP HANA solution
# It is supported by Acronis only as part of the entire solution in accordance with http://www.acronis.com/support/eula-corp.html
# By using this script you accept the terms of the EULA http://www.acronis.com/support/eula-corp.html
# Modification or use outside of the solution is done at your own risk and will not be supported by Acronis
# Contact http://www.acronis.com/en-us/support/ with any questions or concerns
# Copyright (C) 2019 Acronis

import subprocess
import sys

from util import CommandHandler, get_credentials, read_sapservices, Credentials, logger, decrypt

#Main code
def main():
    logger.info("Starting Pre-Freeze")
    sapservices = []
    read_sapservices(sapservices)

    credentials = get_credentials()
    if credentials is None:
        logger.error("Config file not found")
        sys.exit(1)

    #Go through each instance
    for service in sapservices:
        command = CommandHandler(service, "", credentials)

        #Go through each database
        for comm in command._commands:
            command.set_db_command("BACKUP DATA FOR FULL SYSTEM CREATE SNAPSHOT COMMENT 'Acronis Backup Pre-Freeze'")
            try:
                subprocess.check_output(comm, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                logger.error(e.output.decode())
                sys.exit(1)

            command.set_db_command("SELECT BACKUP_ID FROM M_BACKUP_CATALOG WHERE ENTRY_TYPE_NAME = 'data snapshot' AND STATE_NAME = 'prepared'")

            try:
                id = subprocess.check_output(comm, universal_newlines=True, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                logger.error(e.output)
                sys.exit(1)

            logger.info("Instance num: " + command._instance_num)
            logger.info("ID: " + id)
            logger.info("Database: " + comm[command.get_database_position()])
            if id:
                logger.info("Snapshot successful\n\n")
            else:
                logger.error("Snapshot unsuccessful\n\n")
                sys.exit(1)

        logger.info("Instance done\n")

    logger.info("Pre-freeze done\n")

if __name__ == "__main__":
    main()