# -*- coding: utf-8 -*-

import os.path
import sysconfig
import tarfile


GLOBAL_DATA = 'global.dat'
LOCALE_DIR = 'locale-data/'


def raise_no_data_error():
    raise RuntimeError('The babel data files are not available. '
                       'This usually happens because you are using '
                       'a source checkout from Babel and you did '
                       'not build the data files.  Just make sure '
                       'to run "python setup.py import_cldr" before '
                       'installing the library.')


class LocaleArchive():
    def __init__(self):
        self._archive = None

    @property
    def archive(self):
        if not self._archive:
            path_to_archive = os.path.join(sysconfig.get_path('data'), "locale-data.tar.gz")
            if not os.path.isfile(path_to_archive):
                raise_no_data_error()
            self._archive = tarfile.TarFile.open(path_to_archive)
        return self._archive

    def global_data_exists(self):
        try:
            self.archive.getmember(GLOBAL_DATA)
            return True
        except KeyError:
            return False

    def open_global_data(self):
        return self.archive.extractfile(self.archive.getmember(GLOBAL_DATA))

    def locale_exists(self, locale):
        try:
            self.archive.getmember(LOCALE_DIR + locale)
            return True
        except KeyError:
            return False

    def list_locales(self):
        return self.archive.getnames()

    def open_locale(self, locale):
        return self.archive.extractfile(self.archive.getmember(LOCALE_DIR + locale))


archive = LocaleArchive()
